﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

using Platform.Logging.FileLogger;

namespace Platform.Logging.InteractiveLogger
{
    [ProviderAlias("Interactive")]
    public class PlatformInteractiveLoggerProvider : PlatformBatchLoggerProvider<PlatformInteractiveLoggerOptions>
    {
        private PlatformInteractiveLoggerSubscription _Subscription;

        public PlatformInteractiveLoggerProvider( IConfigurationSection configuration )
            : base( configuration )
        {
            _ProcessId = Process.GetCurrentProcess().Id;
            _Subscription = new PlatformInteractiveLoggerSubscription();
        }

        protected override string SectionName => "Interactive";

        private int _ProcessId;

        internal override PlatformBatchLogger<PlatformInteractiveLoggerOptions> CreateLoggerImplementation( string categoryName )
        {
            return new PlatformInteractiveLogger(this, categoryName);
        }

        protected override Task WriteMessagesAsync( IEnumerable<PlatformBatchLoggerMessage> messages, CancellationToken cancellationToken)
        {
            try
            {
                _Subscription.RaiseEvent(this, new PlatformInteractiveLoggerEventArgs { Messages = messages.ToArray() } );
            }
            catch ( Exception ex )
            {
                SimpleFileLogger.WriteLine(Configuration, "{0} Logger Failed to write messages: {1}", SectionName, ex.Message);
            }
            return Task.CompletedTask;
        }
    }
}
