﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using Microsoft.Extensions.Logging;

namespace Platform.Logging
{
    public class PlatformBatchLogger<TLoggerOptions> : ILogger
        where TLoggerOptions : PlatformBatchLoggerOptions, new()
    {
        private readonly PlatformBatchLoggerProvider<TLoggerOptions> _Provider;

        public PlatformBatchLogger( PlatformBatchLoggerProvider<TLoggerOptions> loggerProvider, string categoryName )
        {
            _Provider = loggerProvider;
            Category = categoryName;
            Filter = loggerProvider.GetFilter(categoryName);
        }

        public IDisposable BeginScope<TState>( TState state )
        {
            return null;
        }

        public string Category { get; set; }

        public Func<string, LogLevel, bool> Filter { get; set; }

        public bool IsEnabled(LogLevel logLevel)
        {
            var isenabled = _Provider.ProviderOptions.IsEnabled ?? _Provider.LoggingOptions.IsEnabled ?? false;
            if ( !isenabled ) return false;

            if (logLevel == LogLevel.None)
                return false;

            if (Filter == null)
                return false;

            return Filter( Category, logLevel );
        }

        public void Log<TState>(LogLevel logLevel, EventId eventId, TState state, Exception exception, Func<TState, Exception, string> formatter)
        {
            Log(DateTimeOffset.Now, logLevel, eventId, state, exception, formatter);
        }

        public void Log<TState>(DateTimeOffset timestamp, LogLevel logLevel, EventId eventId, TState state, Exception exception, Func<TState, Exception, string> formatter)
        {
            // check if enabled
            if (!IsEnabled(logLevel))
                return;

            // create message
            var message = new PlatformBatchLoggerMessage
            {
                Timestamp = timestamp.LocalDateTime,
                ThreadId = Thread.CurrentThread.ManagedThreadId,
                Category = Category,
                LogLevel = logLevel,
                EventId = eventId,
                Exception = exception,
                Message = formatter(state, exception)
            };

            // add message
            _Provider.AddMessage(message);
        }
    }
}
